@@ -1,6 +1,6 @@
 /*
   *
-  *     Copyright (c) 2000-2004 Alberto Reggiori <areggiori@webweaving.org>
+  *     Copyright (c) 2000-2006 Alberto Reggiori <areggiori@webweaving.org>
   *                        Dirk-Willem van Gulik <dirkx@webweaving.org>
   *
   * NOTICE
@@ -11,7 +11,7 @@
   *
   *     http://rdfstore.sourceforge.net/LICENSE
   *
-  * $Id: my_compress.h,v 1.2 2004/08/19 18:57:43 areggiori Exp $
+  * $Id: my_compress.h,v 1.4 2006/06/19 10:10:23 areggiori Exp $
   */
 
 #ifndef _MY_COMPRESS_H
@@ -20,7 +20,7 @@
         bits/sizeof(unsigned int) +\
         bits/sizeof(unsigned int)/15/256 \
         )
-#define IDSBUFF (MAXBUFFRLE( MAXRECORDS )+2*sizeof(unsigned int))
+#define IDSBUFF (MAXBUFFRLE( RDFSTORE_MAXRECORDS )+2*sizeof(unsigned int))
 
 #define MAXRUNLENGTH 16*256*256*256 /* too high for average machine I know.... */
 #define MAXVARLENGTH 127 /* up to 127 bytes repeated - this value can be increased *up to* 127 - but watch out efficiency!! */
